

/**
 * @file custom.c
 *
 */

/*********************
 *      INCLUDES
 *********************/
#include <stdio.h>
#include "lvgl.h"
#include "custom.h"

/*********************
 *      DEFINES
 *********************/

/**********************
 *      TYPEDEFS
 **********************/

/**********************
 *  STATIC PROTOTYPES
 **********************/

/**********************
 *  STATIC VARIABLES
 **********************/

/**
 * Create a demo application
 */
void meter_roll1(void);
void custom_init(lv_ui *ui);
static void anim_meterleft_cb(void * var, int32_t v);
static void anim_contleft_cb(void * var, int32_t v);
static void anim_arc_cb(void * var, int32_t v);
void start_init(void);
static void anim_imgA_cb(void * var, int32_t v);

/**
  * @brief  custom application init.
  * @param  ui : the display screen.
  * @retval none
  */
void custom_init(lv_ui *ui)
{
   start_init();
   meter_roll1();
}

/**
  * @brief  3 meters fade in with a delay.
  * @param  none.
  * @retval none
  */
void start_init(void)
{
 lv_obj_fade_in(meter_ui.screen_meter_img_L, 0, 1000);
 lv_obj_fade_in(meter_ui.screen_meter_label_rmp_v, 0, 1000);
 lv_obj_fade_in(meter_ui.screen_meter_label_rmp, 0, 1000);  
 lv_obj_fade_in(meter_ui.screen_meter_cont_left, 0, 1000);

 lv_obj_fade_in(meter_ui.screen_meter_img_R, 0, 500);
 lv_obj_fade_in(meter_ui.screen_meter_cont_right, 0, 500);
 lv_obj_fade_in(meter_ui.screen_meter_label_bt_v, 0, 500);
 lv_obj_fade_in(meter_ui.screen_meter_label_bt, 0, 500);
 lv_obj_fade_in(meter_ui.screen_meter_label_per, 0, 500); 
}

/**
  * @brief  left meters (imgA) animal call back.
  * @param  none.
  * @retval none
  */
static void anim_imgA_cb(void * var, int32_t v)
{

 lv_img_set_zoom(var, 256-90+v);
 lv_img_set_pivot(var, 128, 128);

 lv_img_set_zoom(meter_ui.screen_meter_img_R, 256-90+v);
 lv_img_set_pivot(meter_ui.screen_meter_img_R, 128, 128);

 if(v>60)
 {
   lv_img_set_zoom(meter_ui.screen_meter_img_C, 256-90+v);
   lv_img_set_pivot(meter_ui.screen_meter_img_C, 200, 200);
 }
}

/**
  * @brief  left meters (contleft) animal call back.
  * @param  none.
  * @retval none
  */
static void anim_contleft_cb(void * var, int32_t v)
{
  lv_obj_set_height(var, v);
  lv_obj_align_to(var, meter_ui.screen_meter,LV_ALIGN_BOTTOM_MID, -230, -134);

  static char buf[8];
  lv_snprintf(buf, sizeof(buf), "%d ", (v*25)/100*100);
  lv_label_set_text(meter_ui.screen_meter_label_rmp_v, buf);
}

/**
  * @brief  right meters (contright) animal call back.
  * @param  none.
  * @retval none
  */
static void anim_contright_cb(void * var, int32_t v)
{
  lv_obj_set_height(var,200- v);
  lv_obj_align_to(var, meter_ui.screen_meter,LV_ALIGN_BOTTOM_MID, 318, -130);

  static char buf[4];
  lv_snprintf(buf, sizeof(buf), "%d ", (100-v/2)/5*5);
  lv_label_set_text(meter_ui.screen_meter_label_bt_v, buf);
}

/**
  * @brief  central meters (arc) animal call back.
  * @param  none.
  * @retval none
  */
static void anim_arc_cb(void * var, int32_t v)
{
  lv_arc_set_angles(var, 130, v);
  static char buf[4];
  lv_snprintf(buf, sizeof(buf), "%d ", (534*v-69000)/1000);
  lv_label_set_text(meter_ui.screen_meter_label_kmh_v, buf);
}

/**
  * @brief  3 meters roll init.
  * @param  none.
  * @retval none
  */
void meter_roll1(void)
{

  lv_anim_t a_cont_left,a_cont_right,a_arc_C,a_node;


  lv_anim_init(&a_cont_left);
  lv_anim_set_var(&a_cont_left, meter_ui.screen_meter_cont_left);
  lv_anim_set_repeat_count(&a_cont_left, LV_ANIM_REPEAT_INFINITE);
  lv_anim_set_exec_cb(&a_cont_left, anim_contleft_cb);
  lv_anim_set_playback_delay(&a_cont_left, 300);
  lv_anim_set_playback_time(&a_cont_left, 500);
  lv_anim_set_time(&a_cont_left, 4000);
  lv_anim_set_values(&a_cont_left, 50, 195);
  lv_anim_set_path_cb(&a_cont_left, &lv_anim_path_bounce);
  lv_anim_start(&a_cont_left);

  lv_anim_init(&a_cont_right);
  lv_anim_set_var(&a_cont_right, meter_ui.screen_meter_cont_right);
  lv_anim_set_repeat_count(&a_cont_right, LV_ANIM_REPEAT_INFINITE);
  lv_anim_set_exec_cb(&a_cont_right, anim_contright_cb);
  lv_anim_set_playback_delay(&a_cont_right, 300);
  lv_anim_set_playback_time(&a_cont_right, 100);
  lv_anim_set_time(&a_cont_right, 8000);
  lv_anim_set_values(&a_cont_right, 0, 200);
  lv_anim_start(&a_cont_right);

  lv_anim_init(&a_arc_C);
  lv_anim_set_var(&a_arc_C, meter_ui.screen_meter_arc_C);
  lv_anim_set_repeat_count(&a_arc_C, LV_ANIM_REPEAT_INFINITE);
  lv_anim_set_exec_cb(&a_arc_C, anim_arc_cb);
  lv_anim_set_playback_delay(&a_arc_C, 0);
  lv_anim_set_playback_time(&a_arc_C, 600);
  lv_anim_set_time(&a_arc_C, 3000);
  lv_anim_set_values(&a_arc_C, 130, 392);
  lv_anim_start(&a_arc_C);
}
