/**
  **************************************************************************
  * @file     xmc_lcd_5510.h
  * @version  v2.0.0
  * @date     2020-11-02
  * @brief    xmc_lcd program header
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */  
#ifndef __XMC_LCD_5510_H
#define __XMC_LCD_5510_H

#if LCD_480x800

#include "at32f435_437.h"
#include "at_surf_f437_board_delay.h"
typedef struct
{
  u16 lcd_id;
  void (*xmc_init)(void);
  void (*lcd_init)(void);
  void (*lcd_setblock)(u16 xstart,u16 ystart,u16 xend,u16 yend);
  void (*draw_point)(u16 x,u16 y,u16 color);
  void (*lcd_clear)(u16 color);
}_lcd_dev;

extern _lcd_dev  lcd_dev;
extern _lcd_dev *lcd;

/**
  * @brief lcd display direction
  */
typedef enum
{
  LCD_DISPLAY_HORIZONTAL                  = 0x01, /*!< horizontal display */
  LCD_DISPLAY_VERTICAL                    = 0x00  /*!< vertical display */
} lcd_display_type;  



#define XMC_LCD_COMMAND                  (0x60000000)  
#define XMC_LCD_DATA                     (0x60000000 | 0x800)
/**
  * @brief the lcd io definition
  */
#define LCD_D0_GPIO_PORT                 GPIOD
#define LCD_D0_GPIO_CLK                  CRM_GPIOD_PERIPH_CLOCK
#define LCD_D0_GPIO_PIN                  GPIO_PINS_14
#define LCD_D0_GPIO_PINS_SOURCE          GPIO_PINS_SOURCE14 
#define LCD_D0_GPIO_MUX                  GPIO_MUX_12   

#define LCD_D1_GPIO_PORT                 GPIOD
#define LCD_D1_GPIO_CLK                  CRM_GPIOD_PERIPH_CLOCK
#define LCD_D1_GPIO_PIN                  GPIO_PINS_15
#define LCD_D1_GPIO_PINS_SOURCE          GPIO_PINS_SOURCE15 
#define LCD_D1_GPIO_MUX                  GPIO_MUX_12     
                                         
#define LCD_D2_GPIO_PORT                 GPIOD
#define LCD_D2_GPIO_CLK                  CRM_GPIOD_PERIPH_CLOCK
#define LCD_D2_GPIO_PIN                  GPIO_PINS_0
#define LCD_D2_GPIO_PINS_SOURCE          GPIO_PINS_SOURCE0 
#define LCD_D2_GPIO_MUX                  GPIO_MUX_12     
                                         
#define LCD_D3_GPIO_PORT                 GPIOD
#define LCD_D3_GPIO_CLK                  CRM_GPIOD_PERIPH_CLOCK
#define LCD_D3_GPIO_PIN                  GPIO_PINS_1
#define LCD_D3_GPIO_PINS_SOURCE          GPIO_PINS_SOURCE1 
#define LCD_D3_GPIO_MUX                  GPIO_MUX_12     
                                         
#define LCD_D4_GPIO_PORT                 GPIOE
#define LCD_D4_GPIO_CLK                  CRM_GPIOE_PERIPH_CLOCK
#define LCD_D4_GPIO_PIN                  GPIO_PINS_7
#define LCD_D4_GPIO_PINS_SOURCE          GPIO_PINS_SOURCE7 
#define LCD_D4_GPIO_MUX                  GPIO_MUX_12     
                                         
#define LCD_D5_GPIO_PORT                 GPIOE
#define LCD_D5_GPIO_CLK                  CRM_GPIOE_PERIPH_CLOCK
#define LCD_D5_GPIO_PIN                  GPIO_PINS_8
#define LCD_D5_GPIO_PINS_SOURCE          GPIO_PINS_SOURCE8 
#define LCD_D5_GPIO_MUX                  GPIO_MUX_12     
                                         
#define LCD_D6_GPIO_PORT                 GPIOE
#define LCD_D6_GPIO_CLK                  CRM_GPIOE_PERIPH_CLOCK
#define LCD_D6_GPIO_PIN                  GPIO_PINS_9
#define LCD_D6_GPIO_PINS_SOURCE          GPIO_PINS_SOURCE9 
#define LCD_D6_GPIO_MUX                  GPIO_MUX_12     
                                         
#define LCD_D7_GPIO_PORT                 GPIOE
#define LCD_D7_GPIO_CLK                  CRM_GPIOE_PERIPH_CLOCK
#define LCD_D7_GPIO_PIN                  GPIO_PINS_10
#define LCD_D7_GPIO_PINS_SOURCE          GPIO_PINS_SOURCE10 
#define LCD_D7_GPIO_MUX                  GPIO_MUX_12     
                                         
#define LCD_D8_GPIO_PORT                 GPIOE
#define LCD_D8_GPIO_CLK                  CRM_GPIOE_PERIPH_CLOCK
#define LCD_D8_GPIO_PIN                  GPIO_PINS_11
#define LCD_D8_GPIO_PINS_SOURCE          GPIO_PINS_SOURCE11 
#define LCD_D8_GPIO_MUX                  GPIO_MUX_12     
                                         
#define LCD_D9_GPIO_PORT                 GPIOE
#define LCD_D9_GPIO_CLK                  CRM_GPIOE_PERIPH_CLOCK
#define LCD_D9_GPIO_PIN                  GPIO_PINS_12
#define LCD_D9_GPIO_PINS_SOURCE          GPIO_PINS_SOURCE12 
#define LCD_D9_GPIO_MUX                  GPIO_MUX_12     

#define LCD_D10_GPIO_PORT                GPIOE
#define LCD_D10_GPIO_CLK                 CRM_GPIOE_PERIPH_CLOCK
#define LCD_D10_GPIO_PIN                 GPIO_PINS_13
#define LCD_D10_GPIO_PINS_SOURCE         GPIO_PINS_SOURCE13 
#define LCD_D10_GPIO_MUX                 GPIO_MUX_12     
                                         
#define LCD_D11_GPIO_PORT                GPIOE
#define LCD_D11_GPIO_CLK                 CRM_GPIOE_PERIPH_CLOCK
#define LCD_D11_GPIO_PIN                 GPIO_PINS_14
#define LCD_D11_GPIO_PINS_SOURCE         GPIO_PINS_SOURCE14 
#define LCD_D11_GPIO_MUX                 GPIO_MUX_12     
                                         
#define LCD_D12_GPIO_PORT                GPIOE
#define LCD_D12_GPIO_CLK                 CRM_GPIOE_PERIPH_CLOCK
#define LCD_D12_GPIO_PIN                 GPIO_PINS_15
#define LCD_D12_GPIO_PINS_SOURCE         GPIO_PINS_SOURCE15 
#define LCD_D12_GPIO_MUX                 GPIO_MUX_12     

#define LCD_D13_GPIO_PORT                GPIOD
#define LCD_D13_GPIO_CLK                 CRM_GPIOD_PERIPH_CLOCK
#define LCD_D13_GPIO_PIN                 GPIO_PINS_8
#define LCD_D13_GPIO_PINS_SOURCE         GPIO_PINS_SOURCE8 
#define LCD_D13_GPIO_MUX                 GPIO_MUX_12     
                                         
#define LCD_D14_GPIO_PORT                GPIOD
#define LCD_D14_GPIO_CLK                 CRM_GPIOD_PERIPH_CLOCK
#define LCD_D14_GPIO_PIN                 GPIO_PINS_9
#define LCD_D14_GPIO_PINS_SOURCE         GPIO_PINS_SOURCE9
#define LCD_D14_GPIO_MUX                 GPIO_MUX_12     
                                         
#define LCD_D15_GPIO_PORT                GPIOD
#define LCD_D15_GPIO_CLK                 CRM_GPIOD_PERIPH_CLOCK
#define LCD_D15_GPIO_PIN                 GPIO_PINS_10
#define LCD_D15_GPIO_PINS_SOURCE         GPIO_PINS_SOURCE10
#define LCD_D15_GPIO_MUX                 GPIO_MUX_12 
                                         
#define LCD_NE1_GPIO_PORT                GPIOD
#define LCD_NE1_GPIO_CLK                 CRM_GPIOD_PERIPH_CLOCK
#define LCD_NE1_GPIO_PIN                 GPIO_PINS_7
#define LCD_NE1_GPIO_PINS_SOURCE         GPIO_PINS_SOURCE7
#define LCD_NE1_GPIO_MUX                 GPIO_MUX_12     
                                         
#define LCD_NWE_GPIO_PORT                GPIOD
#define LCD_NWE_GPIO_CLK                 CRM_GPIOD_PERIPH_CLOCK
#define LCD_NWE_GPIO_PIN                 GPIO_PINS_5
#define LCD_NWE_GPIO_PINS_SOURCE         GPIO_PINS_SOURCE5 
#define LCD_NWE_GPIO_MUX                 GPIO_MUX_12     
                                         
#define LCD_NOE_GPIO_PORT                GPIOD
#define LCD_NOE_GPIO_CLK                 CRM_GPIOD_PERIPH_CLOCK
#define LCD_NOE_GPIO_PIN                 GPIO_PINS_4
#define LCD_NOE_GPIO_PINS_SOURCE         GPIO_PINS_SOURCE4 
#define LCD_NOE_GPIO_MUX                 GPIO_MUX_12     
                                         
#define LCD_A10_GPIO_PORT                 GPIOG
#define LCD_A10_GPIO_CLK                  CRM_GPIOG_PERIPH_CLOCK
#define LCD_A10_GPIO_PIN                  GPIO_PINS_0
#define LCD_A10_GPIO_PINS_SOURCE          GPIO_PINS_SOURCE0  
#define LCD_A10_GPIO_MUX                  GPIO_MUX_12     

#define LCD_BL_GPIO_CLK                  
#define LCD_BL_GPIO_PIN                  
#define LCD_BL_GPIO_PORT                 

#define LCD_RESET_GPIO_CLK               
#define LCD_RESET_GPIO_PIN               
#define LCD_RESET_GPIO_PORT              

/* the lcd back light line */
#define LCD_BL_HIGH                   
#define LCD_BL_LOW                    

/* the lcd reset line */                 
#define LCD_RESET_HIGH                 
#define LCD_RESET_LOW     
static u16 LCD_ID;
#define WHITE         	 0xFFFF
#define BLACK         	 0x0000	  
#define BLUE         	   0x001F  
#define BRED             0XF81F
#define GRED 			       0XFFE0
#define GBLUE			       0X07FF
#define RED           	 0xF800
#define MAGENTA       	 0xF81F
#define GREEN         	 0x07E0
#define CYAN          	 0x7FFF
#define YELLOW        	 0xFFE0
#define BROWN 			     0XBC40
#define BRRED 			     0XFC07
#define GRAY  			     0X8430

//ɨ跽
#define L2R_U2D  0 //,ϵ
#define L2R_D2U  1 //,µ
#define R2L_U2D  2 //ҵ,ϵ
#define R2L_D2U  3 //ҵ,µ

#define U2D_L2R  4 //ϵ,
#define U2D_R2L  5 //ϵ,ҵ
#define D2U_L2R  6 //µ,
#define D2U_R2L  7 //µ,ҵ	 

#define DFT_SCAN_DIR  L2R_U2D  //Ĭϵɨ跽

void XMC_init(void);
void LCD_init(u8 dir);
void LCD_WR_COMMAND(u16 command);
void LCD_WR_DATA(u16 data);
u16 LCD_RD_DATA(void);
void LCD_SetBlock(u16 xstart,u16 ystart,u16 xend,u16 yend);
void LCD_WriteOnePoint(u16 color);
void LCD_DrawPoint(u16 x,u16 y,u16 color);
void LCD_Clear(u16 color);
void LCD_WriteReg(u16 LCD_Reg, u16 LCD_RegValue);
void LCD_Set_Scan_Dir(u8 dir);

#endif
#endif


