/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.0
  * @date     2020-11-02
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f435_437_clock.h"
#include "at_surf_f437_board_usart.h"
#include "lv_port_disp_template.h"
#include "lv_port_indev_template.h"
#include "at_surf_f437_board_delay.h"
#include "at_surf_f437_board_sdram.h"
#include "at_surf_f437_board_qspi_sram.h"
#include "at_surf_f437_board_sd_card.h"	
#include "at_surf_f437_board_audio.h"
#include "lv_tick_custom.h"
#include <stdint.h>
#include <string.h>
#include "lvgl.h"
#include <stdio.h>
#include "gui_meter.h"
#include "events_init.h"
#include "custom.h"
#include "xmc_lcd_5510.h"


lv_ui meter_ui;



/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
      int i=0;
int main(void)
{

  system_clock_config();
  delay_init();

  /* usart init for printf debug */   
  uart_print_init(115200);

  
  /* for littlevgl gui tick increase, 1 ms tick */  
  tmr7_int_init(287, 999);

  /* lcd init */  
  LCD_init(LCD_DISPLAY_VERTICAL);
  LCD_Clear(RED);

  /* lvgl gui init */ 
  lv_init();
  lv_port_disp_init();

  /* meter ui config*/ 
  setup_ui(&meter_ui);
  events_init(&meter_ui);
  custom_init(&meter_ui);

  while(1)
  {
    lv_task_handler();   
  }
}



